import formatBytes from "../lib/format-bytes.js";

import defaults from "files-pipeline/dist/options/index.js";
import deepmerge from "files-pipeline/dist/lib/deepmerge.js";

import type SVG from "./svg.js";
import type IMG from "./img.js";
import type JS from "./js.js";
import type HTML from "./html.js";
import type CSS from "./css.js";

import type { Options as OptionsBase } from "files-pipeline/dist/options/index.js";

export interface Options extends OptionsBase {
	// rome-ignore lint/suspicious/noExplicitAny:
	[key: string]: any;

	css?: boolean | CSS;

	html?: boolean | HTML;

	js?: boolean | JS;

	img?: boolean | IMG;

	svg?: boolean | SVG;
}

export default deepmerge(defaults, {
	css: {
		clone: false,
		comments: false,
		forceMediaMerge: true,
	},
	html: {
		caseSensitive: true,
		collapseInlineTagWhitespace: false,
		collapseWhitespace: true,
		continueOnParseError: true,
		html5: true,
		includeAutoGeneratedTags: true,
		keepClosingSlash: true,
		minifyCSS: true,
		minifyJS: true,
		minifyURLs: false,
		preventAttributesEscaping: false,
		processConditionalComments: false,
		removeAttributeQuotes: true,
		removeComments: true,
		ignoreCustomComments: [/^\s*#/, /^\s*\[/, /^\s*\]/, /^\s*!/, /^\s*\//],
		removeScriptTypeAttributes: true,
		removeStyleLinkTypeAttributes: true,
		removeTagWhitespace: false,
		sortAttributes: true,
		sortClassName: true,
		trimCustomFragments: true,
		useShortDoctype: false,
		noNewlinesBeforeTagClose: true,
		quoteCharacter: '"',
	},
	js: {
		// rome-ignore lint/nursery/noPrecisionLoss:
		ecma: 5,
		enclose: false,
		keep_classnames: false,
		keep_fnames: false,
		ie8: false,
		module: false,
		safari10: false,
		toplevel: false,
	},
	img: {
		avif: {
			chromaSubsampling: "4:4:4",
			// rome-ignore lint/nursery/noPrecisionLoss:
			effort: 9,
		},
		gif: {
			// rome-ignore lint/nursery/noPrecisionLoss:
			effort: 10,
		},
		jpeg: {
			chromaSubsampling: "4:4:4",
			mozjpeg: true,
			trellisQuantisation: true,
			overshootDeringing: true,
			optimiseScans: true,
		},
		png: {
			// rome-ignore lint/nursery/noPrecisionLoss:
			compressionLevel: 9,
			palette: true,
		},
		raw: {},
		tiff: {
			compression: "lzw",
		},
		webp: {
			// rome-ignore lint/nursery/noPrecisionLoss:
			effort: 6,
		},
	},
	svg: {
		multipass: true,
		js2svg: {
			indent: 0,
			pretty: false,
		},
		plugins: ["preset-default"],
	},
	pipeline: {
		failed: async (current) =>
			`Error: Cannot compress file ${current.inputPath}!`,
		passed: async (current) =>
			current.fileSizeBefore >
			Buffer.byteLength(current.buffer.toString()),
		accomplished: async (current) =>
			`Compressed ${current.inputPath} for ${await formatBytes(
				current.fileSizeBefore - current.fileSizeAfter
			)} (${(
				((current.fileSizeBefore - current.fileSizeAfter) /
					current.fileSizeBefore) *
				100
			)
				// rome-ignore lint/nursery/noPrecisionLoss:
				.toFixed(2)}% reduction) in ${current.outputPath}.`,
		changed: async (pipe) => {
			pipe.info.total =
				(pipe.info.total ? pipe.info.total : 0) +
				(pipe.current.fileSizeBefore - pipe.current.fileSizeAfter);
			return pipe;
		},
	},
} satisfies Options) as Options;
